/*
 * Copyright (c) 2005 SilverStorm Technologies.  All rights reserved.
 *
 * This software is available to you under the OpenIB.org BSD license
 * below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * $Id: cl_syscallback.c 9 2005-05-23 22:38:08Z ftillier $
 */


#include "complib/cl_syscallback.h"
#include "complib/cl_memory.h"


/* The OS porvides the thread pool, so this is a no-op. */
cl_status_t
cl_sys_callback_init( void )
{
	return( CL_SUCCESS );
}


/* The OS porvides the thread pool, so this is a no-op. */
void
cl_sys_callback_destroy( void )
{
}


/* Note: This could be improved to pull items out of a pool. */
cl_sys_callback_item_t* 
cl_sys_callback_get( 
	IN	const void* const	get_context )
{
	return IoAllocateWorkItem( (PDEVICE_OBJECT)get_context );
}


void 
cl_sys_callback_put( 
	IN	cl_sys_callback_item_t* const	p_item )
{
	CL_ASSERT( p_item );
	IoFreeWorkItem( p_item );
}


cl_status_t 
cl_sys_callback_queue( 
	IN	cl_sys_callback_item_t* const	p_item, 
	IN	cl_pfn_sys_callback_t			pfn_callback,
	IN	const void* const				queue_context, 
	IN	const boolean_t					high_priority )
{
	WORK_QUEUE_TYPE		type;

	/* Execute as an APC if the requested priority is high. */
	if( high_priority )
		type = CriticalWorkQueue;
	else
		type = DelayedWorkQueue;

	IoQueueWorkItem( p_item, pfn_callback, type, (void*)queue_context );

	return( CL_SUCCESS );
}
