/*
 * Copyright (c) 2005 SilverStorm Technologies.  All rights reserved.
 *
 * This software is available to you under the OpenIB.org BSD license
 * below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * $Id: cl_syscallback.c 10 2005-05-24 00:33:03Z ftillier $
 */



#include "complib/cl_syscallback.h"
#include "complib/cl_memory.h"


/* 
 * Callback function invoked by the thread pool in response to a
 * call to QueueUserWorkItem.
 */
static DWORD WINAPI 
cl_sys_callback_wrapper(
	IN	cl_sys_callback_item_t* const	p_item )
{
	CL_ASSERT( p_item );

	p_item->pfn_callback( (void*)p_item->get_context, 
		(void*)p_item->queue_context );

	return( 0 );
}


/* The OS porvides the thread pool, so this is a no-op. */
cl_status_t
cl_sys_callback_init( void )
{
	return( CL_SUCCESS );
}


/* The OS porvides the thread pool, so this is a no-op. */
void
cl_sys_callback_destroy( void )
{
}


/* Note: This could be improved to pull items out of a pool. */
cl_sys_callback_item_t* 
cl_sys_callback_get( 
	IN	const void* const	get_context )
{
	cl_sys_callback_item_t	*p_item;

	p_item = (cl_sys_callback_item_t*)
		cl_zalloc( sizeof(cl_sys_callback_item_t) );

	if( !p_item )
		return( NULL );

	p_item->get_context = get_context;

	return p_item;
}


void 
cl_sys_callback_put( 
	IN	cl_sys_callback_item_t* const	p_item )
{
	CL_ASSERT( p_item );
	cl_free( p_item );
}


cl_status_t 
cl_sys_callback_queue( 
	IN	cl_sys_callback_item_t* const	p_item, 
	IN	cl_pfn_sys_callback_t			pfn_callback,
	IN	const void* const				queue_context, 
	IN	const boolean_t					high_priority )
{
	ULONG	flags;

	/* Store the callback and context in the item */
	p_item->pfn_callback = pfn_callback;
	p_item->queue_context = queue_context;

	/* Execute as an APC if the requested priority is high. */
	if( high_priority )
		flags = WT_EXECUTEINIOTHREAD;
	else
		flags = WT_EXECUTEDEFAULT;

	/* Queue the internal callback with a pointer to the callback item. */
	if( !QueueUserWorkItem( cl_sys_callback_wrapper, p_item, flags ) )
		return( CL_ERROR );

	return( CL_SUCCESS );
}
