/*
 * Copyright (c) 2004, 2005 Topspin Communications.  All rights reserved.
 * Copyright (c) 2005 Sun Microsystems, Inc. All rights reserved.
 * Copyright (c) 2005, 2006, 2007 Cisco Systems.  All rights reserved.
 * Copyright (c) 2005 Mellanox Technologies. All rights reserved.
 * Copyright (c) 2004 Voltaire, Inc. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef MLX4_H
#define MLX4_H

#include <mlx4_debug.h>
#include "l2w.h"
#include "device.h"
#include "doorbell.h"
#include "bus_intf.h"
#include "eq.h"


//
// Structure for reporting data to WMI
//

typedef struct _BUS_WMI_STD_DATA {
	UINT32 DebugPrintLevel;
	UINT32 DebugPrintFlags;

} BUS_WMI_STD_DATA, * PBUS_WMI_STD_DATA;


//
// Driver global data
//

enum mlx4_port_type {
	MLX4_PORT_TYPE_IB	= 1 << 0,
	MLX4_PORT_TYPE_ETH	= 1 << 1,
};

enum mlx4_port_state {
	MLX4_PORT_ENABLED	= 1 << 0,
	MLX4_PORT_DISABLED	= 1 << 1,
};

#define MAX_HCA_CARDS		8

#pragma warning(disable:4201) // nameless struct/union
typedef struct _GLOBALS {
	BUS_WMI_STD_DATA bwsd;

	int mod_num_qp;
	int mod_rdmarc_per_qp;
	int mod_num_srq;
	int mod_num_cq;
	int mod_num_mcg;
	int mod_num_mpt;
	int mod_num_mtt;
	int mod_num_mac;
	int mod_num_vlan;
	int mod_use_prio;
	int mod_num_fc_exch;

	int mod_enable_qos;
	int mod_mlx4_blck_lb;
	int mod_interrupt_from_first;

	int mod_affinity;
	int mod_stat_flags;
	PFDO_DEVICE_DATA p_fdo[MAX_HCA_CARDS];	// for debug purposes
} GLOBALS;
#pragma warning(default:4201) // nameless struct/union

extern GLOBALS g;


enum {
	MLX4_HCR_BASE		= 0x80680,
	MLX4_HCR_SIZE		= 0x0001c,
	MLX4_CLR_INT_SIZE	= 0x00008
};

enum {
	MLX4_MGM_ENTRY_SIZE	=  0x100,
	MLX4_QP_PER_MGM		= 4 * (MLX4_MGM_ENTRY_SIZE / 16 - 2),
	MLX4_MTT_ENTRY_PER_SEG	= 8
};

enum {
	MLX4_EQ_ASYNC,
	MLX4_EQ_COMP,
	MLX4_NUM_EQ
};
#define MLX4_NUM_UNKNOWN	-1

#define MLX4_MAX_EXTRA_EQS 16
#define MLX4_NUM_EQS (MLX4_NUM_EQ + MLX4_MAX_EXTRA_EQS)

enum {
	MLX4_NUM_PDS		= 1 << 15
};

enum {
	MLX4_CMPT_TYPE_QP	= 0,
	MLX4_CMPT_TYPE_SRQ	= 1,
	MLX4_CMPT_TYPE_CQ	= 2,
	MLX4_CMPT_TYPE_EQ	= 3,
	MLX4_CMPT_NUM_TYPE
};

enum {
	MLX4_CMPT_SHIFT		= 24,
	MLX4_NUM_CMPTS		= MLX4_CMPT_NUM_TYPE << MLX4_CMPT_SHIFT
};

#define MGM_QPN_MASK       0x00FFFFFF
#define MGM_BLCK_LB_BIT    30

struct mlx4_bitmap {
	u32			last;
	u32			top;
	u32			max;
	u32			effective_max;
	u32			mask;
	spinlock_t		lock;
	unsigned long	       *table;
};

struct mlx4_buddy {
	unsigned long	      **bits;
	int			max_order;
	spinlock_t		lock;
};

struct mlx4_icm;

struct mlx4_icm_table {
	u64			virt;
	int			num_icm;
	int			num_obj;
	int			obj_size;
	int			lowmem;
	int			coherent;
	struct mutex		mutex;
	struct mlx4_icm	      **icm;
};

struct mlx4_profile {
	int			num_qp;
	int			rdmarc_per_qp;
	int			num_srq;
	int			num_cq;
	int			num_mcg;
	int			num_mpt;
	int			num_mtt;
};

struct mlx4_fw {
	u64			clr_int_base;
	u64			catas_offset;
	struct mlx4_icm	       *fw_icm;
	struct mlx4_icm	       *aux_icm;
	u32			catas_size;
	u16			fw_pages;
	u8			clr_int_bar;
	u8			catas_bar;
};

struct mlx4_cmd {
	struct pci_pool	       *pool;
	u8 __iomem	       *hcr;
	struct mutex		hcr_mutex;
	struct semaphore	poll_sem;
	struct semaphore	event_sem;
	int			max_cmds;
	spinlock_t		context_lock;
	int			free_head;
	struct mlx4_cmd_context *context;
	u16			token_mask;
	u8			use_events;
	u8			toggle;
};

struct mlx4_uar_table {
	struct mlx4_bitmap	bitmap;
};

struct mlx4_mr_table {
	struct mlx4_bitmap	mpt_bitmap;
	struct mlx4_buddy	mtt_buddy;
	u64			mtt_base;
	u64			mpt_base;
	struct mlx4_icm_table	mtt_table;
	struct mlx4_icm_table	dmpt_table;
};

struct mlx4_cq_table {
	struct mlx4_bitmap	bitmap;
	spinlock_t		lock;
	struct radix_tree_root	tree;
	struct mlx4_icm_table	table;
	struct mlx4_icm_table	cmpt_table;
};

struct mlx4_eq_table {
	struct mlx4_bitmap	bitmap;
	void __iomem	       *clr_int;
	u8 __iomem	       *uar_map[(MLX4_NUM_EQS + 6) / 4];
	u32			clr_mask;
	struct mlx4_eq		eq[MLX4_NUM_EQS];
	struct mlx4_icm_table	table;
	struct mlx4_icm_table	cmpt_table;
	int			have_irq;
	u8			inta_pin;
	u8						max_extra_eqs;
#if 1//WORKAROUND_POLL_EQ
	KEVENT		thread_start_event;
	KEVENT		thread_stop_event;
	BOOLEAN		bTerminated;
	PVOID		threadObject;
#endif
};

struct mlx4_srq_table {
	struct mlx4_bitmap	bitmap;
	spinlock_t		lock;
	struct radix_tree_root	tree;
	struct mlx4_icm_table	table;
	struct mlx4_icm_table	cmpt_table;
};

struct mlx4_qp_table {
	struct mlx4_bitmap	bitmap;
	u32			rdmarc_base;
	int			rdmarc_shift;
	spinlock_t		lock;
	struct mlx4_icm_table	qp_table;
	struct mlx4_icm_table	auxc_table;
	struct mlx4_icm_table	altc_table;
	struct mlx4_icm_table	rdmarc_table;
	struct mlx4_icm_table	cmpt_table;
};

struct mlx4_mcg_table {
	struct mutex		mutex;
	struct mlx4_bitmap	bitmap;
	struct mlx4_icm_table	table;
};

struct mlx4_catas_err {
	u32 __iomem	       *map;
	/* Windows */
	int					stop;
	KTIMER				timer;
	KDPC				timer_dpc;
	LARGE_INTEGER		interval;
	PIO_WORKITEM		catas_work;
};

struct mlx4_mac_table {
#define MLX4_MAX_MAC_NUM	128
#define MLX4_MAC_MASK		0xffffffffffff
#define MLX4_MAC_VALID_SHIFT	63
#define MLX4_MAC_TABLE_SIZE	MLX4_MAX_MAC_NUM << 3
	__be64 entries[MLX4_MAX_MAC_NUM];
	int refs[MLX4_MAX_MAC_NUM];
	struct semaphore mac_sem;
	int total;
	int max;
};

struct mlx4_vlan_table {
#define MLX4_MAX_VLAN_NUM	126
#define MLX4_VLAN_MASK		0xfff
#define MLX4_VLAN_VALID		1 << 31
#define MLX4_VLAN_TABLE_SIZE	MLX4_MAX_VLAN_NUM << 2
	__be32 entries[MLX4_MAX_VLAN_NUM];
	int refs[MLX4_MAX_VLAN_NUM];
	struct semaphore vlan_sem;
	int total;
	int max;
};

struct mlx4_port_info {
	struct mlx4_mac_table	mac_table;
	struct mlx4_vlan_table	vlan_table;
};

struct mlx4_priv {
	struct mlx4_dev		dev;

	struct list_head	dev_list;
	struct list_head	ctx_list;
	spinlock_t		ctx_lock;

	struct list_head        pgdir_list;
	struct mutex            pgdir_mutex;

	struct mlx4_fw		fw;
	struct mlx4_cmd		cmd;

	struct mlx4_bitmap	pd_bitmap;
	struct mlx4_uar_table	uar_table;
	struct mlx4_mr_table	mr_table;
	struct mlx4_cq_table	cq_table;
	struct mlx4_eq_table	eq_table;
	struct mlx4_srq_table	srq_table;
	struct mlx4_qp_table	qp_table;
	struct mlx4_mcg_table	mcg_table;

	struct mlx4_catas_err	catas_err;

	u8 __iomem	       *clr_base;

	struct mlx4_uar		driver_uar;
	void __iomem	       *kar;
	struct mlx4_port_info	port[MLX4_MAX_PORTS];
};

static inline struct mlx4_priv *mlx4_priv(struct mlx4_dev *dev)
{
	return container_of(dev, struct mlx4_priv, dev);
}

u32 mlx4_bitmap_alloc(struct mlx4_bitmap *bitmap);
void mlx4_bitmap_free(struct mlx4_bitmap *bitmap, u32 obj);
u32 mlx4_bitmap_alloc_range(struct mlx4_bitmap *bitmap, int cnt, int align);
void mlx4_bitmap_free_range(struct mlx4_bitmap *bitmap, u32 obj, int cnt);
int mlx4_bitmap_init(struct mlx4_bitmap *bitmap, u32 num, u32 mask, u32 reserved);
int mlx4_bitmap_init_with_effective_max(struct mlx4_bitmap *bitmap,
					u32 num, u32 mask, u32 reserved,
					u32 effective_max);
void mlx4_bitmap_cleanup(struct mlx4_bitmap *bitmap);

int mlx4_db_alloc(struct mlx4_dev *dev, 
				struct mlx4_db *db, int order);

int mlx4_init_pd_table(struct mlx4_dev *dev);
int mlx4_init_uar_table(struct mlx4_dev *dev);
int mlx4_init_mr_table(struct mlx4_dev *dev);
int mlx4_init_eq_table(struct mlx4_dev *dev);
int mlx4_init_cq_table(struct mlx4_dev *dev);
int mlx4_init_qp_table(struct mlx4_dev *dev);
int mlx4_init_srq_table(struct mlx4_dev *dev);
int mlx4_init_mcg_table(struct mlx4_dev *dev);

void mlx4_cleanup_pd_table(struct mlx4_dev *dev);
void mlx4_cleanup_uar_table(struct mlx4_dev *dev);
void mlx4_cleanup_mr_table(struct mlx4_dev *dev);
void mlx4_cleanup_eq_table(struct mlx4_dev *dev);
void mlx4_cleanup_cq_table(struct mlx4_dev *dev);
void mlx4_cleanup_qp_table(struct mlx4_dev *dev);
void mlx4_cleanup_srq_table(struct mlx4_dev *dev);
void mlx4_cleanup_mcg_table(struct mlx4_dev *dev);

int mlx4_start_catas_poll(struct mlx4_dev *dev);
void mlx4_stop_catas_poll(struct mlx4_dev *dev);
int mlx4_restart_one(struct pci_dev *pdev);
int mlx4_register_device(struct mlx4_dev *dev);
void mlx4_unregister_device(struct mlx4_dev *dev);
void mlx4_dispatch_event(struct mlx4_dev *dev, enum mlx4_event type,
			 int subtype, int port);
void mlx4_intf_init();
void mlx4_net_init();

BOOLEAN mlx4_is_eth_port(struct mlx4_dev *dev, int port_number);
BOOLEAN mlx4_is_ib_port(struct mlx4_dev *dev, int port_number);
BOOLEAN mlx4_is_enabled_port(struct mlx4_dev *dev, int port_number);
int mlx4_count_ib_ports(struct mlx4_dev *dev);

struct mlx4_dev_cap;
struct mlx4_init_hca_param;

u64 mlx4_make_profile(struct mlx4_dev *dev,
		      struct mlx4_profile *request,
		      struct mlx4_dev_cap *dev_cap,
		      struct mlx4_init_hca_param *init_hca);

int mlx4_cmd_init(struct mlx4_dev *dev);
void mlx4_cmd_cleanup(struct mlx4_dev *dev);
void mlx4_cmd_event(struct mlx4_dev *dev, u16 token, u8 status, u64 out_param);
int mlx4_cmd_use_events(struct mlx4_dev *dev);
void mlx4_cmd_use_polling(struct mlx4_dev *dev);

int mlx4_qp_get_region(struct mlx4_dev *dev,
		       enum qp_region region,
		       int *base_qpn, int *cnt);

void mlx4_cq_completion(struct mlx4_dev *dev, u32 cqn);
void mlx4_cq_event(struct mlx4_dev *dev, u32 cqn, int event_type);

void mlx4_init_mac_table(struct mlx4_dev *dev, u8 port);
void mlx4_init_vlan_table(struct mlx4_dev *dev, u8 port);

void mlx4_qp_event(struct mlx4_dev *dev, u32 qpn, int event_type);

void mlx4_srq_event(struct mlx4_dev *dev, u32 srqn, int event_type);

void mlx4_handle_catas_err(struct mlx4_dev *dev);

int mlx4_init_one(struct pci_dev *pdev, struct mlx4_dev_params *dev_params);

void mlx4_remove_one(struct pci_dev *pdev, int reset);

#define ETH_FCS_LEN	4		/* Frame Check Sequence Length   */
#define ETH_HLEN 14

int mlx4_add_eq(struct mlx4_dev *dev, int nent,
	KAFFINITY cpu, PISR_FUNC func, PVOID func_context ,
	u8* p_eq_num, struct mlx4_eq ** p_eq);

void mlx4_remove_eq(struct mlx4_dev *dev, u8 eq_num);

int mlx4_reset_ready( struct ib_event_handler *event_handler );
int mlx4_reset_execute( struct ib_event_handler *event_handler );

int mlx4_reset_request( struct ib_event_handler *event_handler );

int mlx4_reset_cb_register( struct ib_event_handler *event_handler );

int mlx4_reset_cb_unregister( struct ib_event_handler *event_handler );

void fix_bus_ifc(struct pci_dev *pdev);

void mlx4_dispatch_reset_event(struct ib_device *ibdev, enum ib_event_type type);


#endif /* MLX4_H */
