#pragma once

#ifndef L2W_H
#define L2W_H

////////////////////////////////////////////////////////
//
// GENERAL INCLUDES
//
////////////////////////////////////////////////////////

// OS
#include <ntddk.h>
//#include <iointex.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <errno.h>
#define NTSTRSAFE_LIB
#include <ntstrsafe.h>

// complib
#include <complib/cl_timer.h>
#include <complib/cl_qlist.h>

// mlx4
#include "vc.h"


////////////////////////////////////////////////////////
//
// LITERALS
//
////////////////////////////////////////////////////////

#define BITS_PER_LONG		32
#define N_BARS				3
#define HZ					1000000 /* 1 sec in usecs */
#define EOPNOTSUPP			95


////////////////////////////////////////////////////////
//
// SUBSTITUTIONS
//
////////////////////////////////////////////////////////

#define BUG_ON(exp)		ASSERT(!(exp)) /* in Linux follows here panic() !*/ 
#define snprintf		_snprintf
#define printk			cl_dbg_out
#define KERN_ERR		"err:"
#define KERN_WARNING	"warn:"
#define KERN_DEBUG		"dbg:"

// memory barriers
#define wmb KeMemoryBarrier
#define rmb KeMemoryBarrier
#define mb KeMemoryBarrier
// TODO: can we make it empty ?  I saw in Linux, it is an empty macro for x86 & x64
#define mmiowb KeMemoryBarrier

// linker 
#define EXPORT_SYMBOL_GPL(a)

// gcc compiler attributes
#define __devinit
#define __devinitdata
#define __init
#define __exit
#define __force
#define __iomem
#define __attribute_const__
#define likely(x)			(x)
#define unlikely(x)			(x)
#define __attribute__(a)
#define __bitwise

// container_of
#define container_of		CONTAINING_RECORD

// inline 
#define inline	__inline

// new Linux event mechanism
#define complete(a)					wake_up(a)

// convert
#define __constant_htons		CL_HTON16
#define __constant_cpu_to_be32	CL_HTON32

// various
#define __always_inline				inline

#if (WINVER < _WIN32_WINNT_WIN6)
#define num_possible_cpus() KeNumberProcessors
#else
#define num_possible_cpus()	KeQueryMaximumProcessorCount()
#endif


////////////////////////////////////////////////////////
//
// TYPES
//
////////////////////////////////////////////////////////

// basic types
typedef unsigned char			u8, __u8;
typedef unsigned short int	u16, __u16;
typedef unsigned int				u32, __u32;
typedef unsigned __int64		u64, __u64;
typedef char			s8, __s8;
typedef short int	s16, __s16;
typedef int				s32, __s32;
typedef __int64		s64, __s64;

// inherited
typedef u16  __le16;
typedef u16  __be16;
typedef u32  __le32;
typedef u32  __be32;
typedef u64  __le64;
typedef u64  __be64;
typedef u64 io_addr_t;

// dummy function
typedef void (*MT_EMPTY_FUNC)();

// PCI BAR descriptor
typedef enum _hca_bar_type
{
	HCA_BAR_TYPE_HCR,
	HCA_BAR_TYPE_UAR,
	HCA_BAR_TYPE_DDR,
	HCA_BAR_TYPE_MAX

}	hca_bar_type_t;


typedef struct _hca_bar
{
	uint64_t			phys;
	void				*virt;
	SIZE_T				size;

}	hca_bar_t;

struct msix_saved_info {
	PVOID	vca;		/* MSI-X Vector Table card address */
	PVOID	mca;		/* MSI-X Mask Table card address */
	PVOID	vsa;		/* MSI-X Vector Table saved address */
	PVOID	msa;		/* MSI-X Mask Table saved address */
	ULONG	vsz;		/* MSI-X Vector Table size */
	ULONG	msz;		/* MSI-X Mask Table size */
	int		num;		/* number of supported MSI-X vectors */
	int		valid;		/* the structure is valid */
};

struct msix_map {
	KAFFINITY	cpu;		/* affinity of this MSI-X vector */
	int			eq_ix;		/* EQ index in the array of EQs */
	int			ref_cnt;	/* number of users */
};

typedef struct _MLX4_ST_DEVICE *PMLX4_ST_DEVICE;

// interface structure between Upper and Low Layers of the driver
struct pci_dev
{
	// driver: OS/platform resources
	BUS_INTERFACE_STANDARD			bus_pci_ifc;
	PCI_COMMON_CONFIG				pci_cfg_space;
	struct msix_saved_info			msix_info;
	struct msix_map*				p_msix_map;
	uplink_info_t					uplink_info;
	// driver: card resources
	hca_bar_t						bar[N_BARS];
	CM_PARTIAL_RESOURCE_DESCRIPTOR	int_info;	/* HCA interrupt resources */
	// driver: various objects and info
	USHORT							ven_id;
	USHORT							dev_id;
	USHORT							sub_vendor_id;
	USHORT							sub_system_id;
	UCHAR							revision_id;
	DMA_ADAPTER		*				p_dma_adapter;	/* HCA adapter object */
	DEVICE_OBJECT	*				p_self_do;		/* mlx4_bus's FDO */
	DEVICE_OBJECT	*				pdo;			/* mlx4_bus's PDO */
	PVOID                           p_wdf_device;   /* wdf_device */
	LONG							ib_hca_created;
	// mlx4_ib: various objects and info	
	struct ib_device *				ib_dev;
	// mlx4_net: various objects and info	
	struct mlx4_dev *				dev;
	volatile long					dpc_lock;
	PUCHAR							vpd;
	int								vpd_size;
	WCHAR							location[36];		/* bus+func+dev */
#ifdef USE_WDM_INTERRUPTS
	PKINTERRUPT						int_obj;		/* HCA interrupt object */
	KSPIN_LOCK						isr_lock; 		/* lock for the ISR */
	// MSI-X interrupts
	u8								n_msi_vectors_alloc;/* number of allocated MSI vectors */
	u8								n_msi_vectors;		/* number of MSI vectors; 0 - no MSI */
	ULONG							version;
	int 							legacy_connect;
#endif	
	// statistics
	PMLX4_ST_DEVICE 				p_stat_dev;
};

/* DPC */
typedef void (*dpc_t)( struct _KDPC *, PVOID, PVOID, PVOID );


////////////////////////////////////////////////////////
//
// MACROS
//
////////////////////////////////////////////////////////

// conversions
#define swab32(a)			_byteswap_ulong((ULONG)(a))
#define cpu_to_be16(a)		_byteswap_ushort((USHORT)(a))
#define be16_to_cpu(a)		_byteswap_ushort((USHORT)(a))
#define cpu_to_be32(a)		_byteswap_ulong((ULONG)(a))
#define be32_to_cpu(a)		_byteswap_ulong((ULONG)(a))
#define cpu_to_be64(a)		_byteswap_uint64((UINT64)(a))
#define be64_to_cpu(a)		_byteswap_uint64((UINT64)(a))
#define be64_to_cpup(p)		_byteswap_uint64(*(PUINT64)(p))
#define be32_to_cpup(p)		_byteswap_ulong(*(PULONG)(p))
#define be16_to_cpup(p)		_byteswap_ushort(*(PUSHORT)(p))

// ARRAY_SIZE
#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))

// ALIGN
#define ALIGN(x,a) (((x)+(a)-1)&~((a)-1))
#define PTR_ALIGN(size)	(((size) + sizeof(void*) - 1) & ~(sizeof(void*) - 1))

// there is a bug in Microsoft compiler, that when _byteswap_uint64() gets an expression
// it executes the expression but doesn't swap tte dwords
// So, there's a workaround
#ifdef BYTESWAP_UINT64_BUG_FIXED
#define CPU_2_BE64_PREP		
#define CPU_2_BE64(x)			cl_hton64(x)
#else
#define CPU_2_BE64_PREP	unsigned __int64 __tmp__	
#define CPU_2_BE64(x)			( __tmp__ = x, cl_hton64(__tmp__) )
#endif

#define ERR_PTR(error)		((void*)(LONG_PTR)(error))
#define PTR_ERR(ptr)			((long)(LONG_PTR)(void*)(ptr))
//TODO: there are 2 assumptions here:
// - pointer can't be too big (around -1)
// - error can't be bigger than 1000
#define IS_ERR(ptr)				((ULONG_PTR)ptr > (ULONG_PTR)-1000L)

#define BITS_TO_LONGS(bits) \
	 (((bits)+BITS_PER_LONG-1)/BITS_PER_LONG)

#ifndef ETIMEDOUT
#define ETIMEDOUT		(110)
#endif

#ifdef PAGE_ALIGN
#undef PAGE_ALIGN
#define PAGE_ALIGN(Va) ((u64)((ULONG_PTR)(Va) & ~(PAGE_SIZE - 1)))
#endif

#define NEXT_PAGE_ALIGN(addr)	(((addr)+PAGE_SIZE-1)&PAGE_MASK)

/* typed minimum */
#define min_t(type,x,y)		((type)(x) < (type)(y) ? (type)(x) : (type)(y))

#define DIV_ROUND_UP(n,d) (((n) + (d) - 1) / (d))

#define EXPORT_SYMBOL(name)
#ifndef USE_WDM_INTERRUPTS
#define free_irq(pdev)
#endif

static inline NTSTATUS errno_to_ntstatus(int err)
{
#define MAP_ERR(err,ntstatus)	case err: status = ntstatus; break
	NTSTATUS status;

	if (!err) 
		return STATUS_SUCCESS;

	if (err < 0)
		err = -err;
	switch (err) {
		MAP_ERR( ENOENT, STATUS_NOT_FOUND );
		MAP_ERR( EAGAIN, STATUS_DEVICE_BUSY );
		MAP_ERR( ENOMEM, STATUS_NO_MEMORY );
		MAP_ERR( EACCES, STATUS_ACCESS_DENIED );
		MAP_ERR( EFAULT, STATUS_DRIVER_INTERNAL_ERROR );
		MAP_ERR( EBUSY, STATUS_INSUFFICIENT_RESOURCES );
		MAP_ERR( ENODEV, STATUS_NOT_SUPPORTED );
		MAP_ERR( EINVAL, STATUS_INVALID_PARAMETER );
		MAP_ERR( ENOSYS, STATUS_NOT_SUPPORTED );
		default:
			status = STATUS_UNSUCCESSFUL;
			break;
	}
	return status;
}


////////////////////////////////////////////////////////
//
// PROTOTYPES
//
////////////////////////////////////////////////////////

SIZE_T strlcpy(char *dest, const void *src, SIZE_T size);
int core_init();
void core_cleanup();


////////////////////////////////////////////////////////
//
// SPECIFIC INCLUDES
//
////////////////////////////////////////////////////////

struct mlx4_dev;
struct mlx4_priv;

#include <l2w_atomic.h>
#include <l2w_bit.h>
#include <l2w_bitmap.h>
#include "l2w_debug.h"
#include <l2w_memory.h>
#include <l2w_umem.h>
#include <l2w_list.h>
#include <l2w_pci.h>
#include <l2w_pcipool.h>
#include "l2w_radix.h"
#include <l2w_spinlock.h>
#include <l2w_sync.h>
#include <l2w_time.h>

#include "device.h"

static inline int mlx4_is_livefish(struct mlx4_dev *dev)
{
	if (dev == NULL) {
		return TRUE;
	}
	return !!(dev->flags & MLX4_FLAG_LIVEFISH);
}

static inline int mlx4_is_barred(struct mlx4_dev *dev)
{
	return dev->flags &  MLX4_FLAG_RESET_DRIVER;
}

static inline int mlx4_is_in_reset(struct mlx4_dev *dev)
{
	return dev->flags & MLX4_FLAG_RESET_STARTED;
}

#endif
