#pragma once

// ===========================================
// LITERALS
// ===========================================

#define DEVID_HERMON_SDR			0x6340	/* 25408 */
#define DEVID_HERMON_DDR			0x634a	/* 25418 */
#define DEVID_HERMON_DDR_G2			0x6732	/* 26418 */
#define DEVID_HERMON_DDR_G2_A		0x6778	/* 26488 */
#define DEVID_HERMON_QDR_G2			0x673c	/* 26428 */
#define DEVID_HERMON_QDR_G2_A		0x6746	/* 26438 */

#define DEVID_HERMON_ETH			0x6368	/* 25448 */
#define DEVID_HERMON_ETH_YATIR		0x6372	/* 25458 */
#define DEVID_HERMON_ETH_G2			0x6750	/* 26448 */
#define DEVID_HERMON_ETH_YATIR_G2	0x675A	/* 26458 */
#define DEVID_HERMON_ETH_B0_G2		0x6764	/* 26468 */
#define DEVID_HERMON_ETH_B0_40Gb_G2	0x676E	/* 26478 */
/* livefish */
#define DEVID_HERMON_BD		0x0191	/* 401 */

/* Types of supported HCA */
typedef enum __hca_type {
	HERMON,			/* fully functional HCA */
	LIVEFISH		/* a burning device */
} hca_type_t;

/* vendors */
#define PCI_VENDOR_ID_MELLANOX				0x15b3
#define PCI_VENDOR_ID_TOPSPIN				0x1867

#define HCA(v, d, t) \
	{ PCI_VENDOR_ID_##v,	DEVID_HERMON_##d, t }

struct pci_device_id {
	USHORT		vendor;
	USHORT		device;
	hca_type_t	driver_data;
};


// ===========================================
// TYPES
// ===========================================


// ===========================================
// MACROS/FUNCTIONS
// ===========================================

NTSTATUS pci_hca_reset( struct pci_dev *pdev);

/* use shim to implement that */
#define mlx4_reset(dev)		pci_hca_reset(dev->pdev)

// get bar boundaries
#define pci_resource_start(dev,bar_num)	((dev)->bar[bar_num >> 1].phys)
#define pci_resource_len(dev,bar_num)	((dev)->bar[bar_num >> 1].size)

// i/o to registers

static inline u64 readq(const volatile void __iomem *addr)
{
	//TODO: write atomic implementation of _IO_READ_QWORD and change mthca_doorbell.h
	u64 val;
	READ_REGISTER_BUFFER_ULONG((PULONG)(addr), (PULONG)&val, 2 );
	return val;
}

static inline u32 readl(const volatile void __iomem *addr)
{
	return READ_REGISTER_ULONG((PULONG)(addr));
}

static inline u16 reads(const volatile void __iomem *addr)
{
	return READ_REGISTER_USHORT((PUSHORT)(addr));
}

static inline u8 readb(const volatile void __iomem *addr)
{
	return READ_REGISTER_UCHAR((PUCHAR)(addr));
}

#define __raw_readq		readq
#define __raw_readl		readl
#define __raw_reads		reads
#define __raw_readb		readb

static inline void writeq(unsigned __int64 val, volatile void __iomem *addr)
{
	//TODO: write atomic implementation of _IO_WRITE_QWORD and change mthca_doorbell.h
	WRITE_REGISTER_BUFFER_ULONG( (PULONG)(addr), (PULONG)&val, 2 );
}

static inline void writel(unsigned int val, volatile void __iomem *addr)
{
	WRITE_REGISTER_ULONG((PULONG)(addr),val);
}

static inline void writes(unsigned short val, volatile void __iomem *addr)
{
	WRITE_REGISTER_USHORT((PUSHORT)(addr),val);
}

static inline void writeb(unsigned char val, volatile void __iomem *addr)
{
	WRITE_REGISTER_UCHAR((PUCHAR)(addr),val);
}

#define __raw_writeq		writeq
#define __raw_writel		writel
#define __raw_writes		writes
#define __raw_writeb		writeb


